// POV Clock - Nedelcu Vlad Costin - 336CB

#include <avr/io.h>
#define F_CPU 16000000
#include <util/delay.h>
#include <avr/interrupt.h>
#include <stdio.h>
#include <string.h>

// dimensiunea unui pattern
#define dimension 5

// codificarea punctului separator
#define dot 0x14

// pattern-urile de LED-uri ce vor forma cifra dorita
unsigned char patterns[10][dimension] = {
                                   {0x7f, 0x41, 0x41, 0x41, 0x7f}, //0
                                   {0x00, 0x00, 0x7f, 0x00, 0x00}, //1
                                   {0x79, 0x49, 0x49, 0x49, 0x4f}, //2
                                   {0x49, 0x49, 0x49, 0x49, 0x7f}, //3
                                   {0x0f, 0x08, 0x08, 0x08, 0x7f}, //4
                                   {0x4f, 0x49, 0x49, 0x49, 0x79}, //5
                                   {0x7f, 0x49, 0x49, 0x49, 0x79}, //6
                                   {0x01, 0x01, 0x01, 0x01, 0x7f}, //7
                                   {0x7f, 0x49, 0x49, 0x49, 0x7f}, //8
                                   {0x4f, 0x49, 0x49, 0x49, 0x7f} //9
			                        };

// variabilele pentru contorizarea timpului
volatile unsigned char minut = 0x00, ora=0x00, secunda=0x00;

// setarea orei curente
void setClock(unsigned char hour,unsigned char minute, unsigned char second) {
    ora = hour;
	minut = minute;
	secunda = second;
}

// incrementarea orei curente cu o secunda
void changeTime() {
    secunda++;

	if (secunda == 60){
	    minut++;
		secunda = 0;
		
		if (minut == 60 ){
			ora++;
			minut=0;
			
			if (ora == 12 )
			    ora = 0;
	    }
	}
}

// handler de intrerupere pentru modificarea orei
ISR(TIMER1_COMPA_vect){
    changeTime();
}

// initializarile LED-urilor
void initBoard(){
    // setare port D ca output
	DDRD = 0xff;

	// setarea LED-urilor ca stinse
	PORTD = 0x00;
}

// initializare timer pentru intrerupere la o secunda
void initTimer(){
    // FPWM cu top la OCR1A, PS 1024
    TCCR1B = (3 << WGM12) | (5 << CS10);
    TCCR1A = (3 << WGM10);
    
    TIMSK1 = (1 << OCIE1A);
    OCR1A = 18825;
}

// afisarea unui pattern
void displayPattern(unsigned char* pattern, unsigned char dim) {
	unsigned char j;
	for (j = 0; j < dim; j++) {
		PORTD = pattern[j];

		// adaugare de delay-uri pentru afisarea cat mai clara a cifrelor
		_delay_ms(2);
		_delay_us(500);
	}
}

int main() {
    // initializarile corespunzatoare (LED-uri, ora, intreruperi)
	initBoard();

	setClock(11,42,0);
	
	initTimer();
	
	sei();

    // bucla principala, se afiseaza fiecare caracter din componenta orei
    // curente(ora:minut:secunda), urmat de o scurta perioada de timp in
    // care se sting toate LED-urile (un fel de spatiu intre ele)
    while(1) {
        displayPattern(patterns[ora/10], dimension);
	         
        PORTD = 0x00;
        _delay_ms(3);
	         
        displayPattern(patterns[ora%10], dimension);
	
        PORTD = 0x00;
        _delay_ms(2);
	
        PORTD = dot;
        _delay_ms(3);
	    
        PORTD = 0x00;
        _delay_ms(2);
	
        displayPattern(patterns[minut/10], dimension);

        PORTD = 0x00;
        _delay_ms(3);
	         
        displayPattern(patterns[minut%10], dimension);
	
        PORTD = 0x00;
        _delay_ms(2);
	
        PORTD = dot;
        _delay_ms(3);
	
        PORTD = 0x00;
        _delay_ms(2);

        displayPattern(patterns[secunda/10], dimension);
	         
        PORTD = 0x00;
        _delay_ms(3);
	         
        displayPattern(patterns[secunda%10], dimension);

        PORTD = 0x00;
        _delay_ms(30);

    }
	
	return 1;
}
